package com.aselimov.app.timedata;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.locks.ReentrantLock;
import java.util.function.Consumer;

import com.aselimov.app.steady_state_analysis.SteadyStateCalculator;

public class TimeData implements Consumer<TimeData.TimeDataPoint> {

    private TimeDataPoints data = new TimeDataPoints();
    private ReentrantLock mutex = new ReentrantLock();

    /**
     * Single time data point.
     * Used for formatted parsing of incoming JSON data
     *
     */
    public record TimeDataPoint(double time, double value) {
    };

    /**
     * Collection of data points
     */
    public class TimeDataPoints {
        public List<Double> times;
        public List<Double> values;

        /**
         * Empty constructor
         */
        public TimeDataPoints() {
            times = new ArrayList();
            values = new ArrayList();
        }

        /**
         * Implement copy constructors for TimeDataPoints
         */
        public TimeDataPoints(TimeDataPoints data) {
            this(data.times, data.values);
        }

        public TimeDataPoints(List<Double> times, List<Double> values) {
            this.times = new ArrayList(times);
            this.values = new ArrayList(values);
        }

        public void add(double time, double value) {
            times.add(time);
            values.add(value);
        }
    };

    /**
     * Allow TimeData to be a consumer of TimeDataPoint.
     * Add the TimeDataPoint to the overall time_data
     */
    @Override
    public void accept(TimeDataPoint arg0) {
        mutex.lock();
        try {
            data.add(arg0.time(), arg0.value());
        } finally {
            mutex.unlock();
        }

    }

    /**
     * Get a clone of the TimeDataPoints.
     * We get a clone for local operation while ensuring the memory access is
     * threadsafe
     */
    public TimeDataPoints threadsafeClone() {
        mutex.lock();
        try {
            return new TimeDataPoints(data.times, data.values);
        } finally {
            mutex.unlock();
        }

    }

    /**
     * Return the number of data points in the class
     * 
     * @return number of data points
     */
    public int getNumPnts() {
        return data.times.size();
    }
}
